﻿(function($) {
    $(function() {
        var ColorPicker = function (container, input, id) {
            this._container = container;
            this._input = input;
            this._id = id;

            this._model = null;

            this._kendoInputType = null;
            this._isKendoInput = false;
        };
        ColorPicker.prototype = {
            // public methods
            init: function () {},
            set_model: function (model) {
                this._model = model;

                if (this._model.ReadOnly) {
                    return;
                }

                $(this.get_input()).kendoColorPicker();
                this._isKendoInput = true;
                this._kendoInputType = "kendoColorPicker";
            },

            get_model: function () { return this._model; },

            add_valueChanged: function (fn) {
                if (typeof (fn) === "function") {
                    var fnDelegate = Function.createDelegate(this, fn);
                    if (this._isKendoInput === true) {
                        this.get_kendoInput().bind("change", fnDelegate);
                    } else {
                        this.get_input().bind("change", fnDelegate);
                    }
                    return fnDelegate;
                }
            },

            remove_valueChanged: function(fnd) {
                if (this._isKendoInput === true) {
                    this.get_kendoInput().unbind("change", fnd);
                } else {
                    this.get_input().unbind("change", fnd);
                }
            },

            hide: function () { $(this.get_container()).addClass("hide"); },
            show: function () { $(this.get_container()).removeClass("hide"); },
            toggle: function () { $(this.get_container()).toggleClass("hide"); },

            enable: function () {
                if (this._isKendoInput) {
                    this.get_kendoInput().enable(true);
                }
                this.get_input().attr('disabled', false);
            },
            disable: function () {
                if (this._isKendoInput) {
                    this.get_kendoInput().enable(false);
                }
                this.get_input().attr('disabled', true);
            },

            readonly: function (value) {
                if (this.get_model().ReadOnly)
                    return true;
                throw "Changing the readonly status for \"" + this.get_input()[0].name + "\" field has not been enabled";
            },
            required: function (value) {
                throw "Changing the required status for \"" + this.get_input()[0].name + "\" field has not been enabled";
            },

            clear: function () {
                if (this._model.ReadOnly)
                    throw "Cannot clear value of readonly field";
                if (this._isKendoInput) {
                    this.get_kendoInput().value("");
                } else {
                    this.get_input().val("");
                }
            },
            // getters & setters
            set_value: function(value) {
                if (this._model.ReadOnly) {
                    throw "Cannot set value of readonly text field";
                } else {
                    if (this._isKendoInput) this.get_kendoInput().value(value);
                    else $(this.get_input()).val(value);
                }
            },
            get_value: function() {
                if (this.get_model().ReadOnly) {
                    return $(this.get_input()).data("tpc-readonly-value");
                }

                if (this._isKendoInput) {
                    var val = this.get_kendoInput().value();
                    if (val === null) {
                        return "";
                    }
                    return val;}

                return $(this.get_input()).val();
            },           

            get_kendoInput: function() {
                if (this._model.ReadOnly) {
                    throw "Kendo input not defined on readonly text field";
                } else if (this._isKendoInput) {
                    return $(this.get_input()).data(this._kendoInputType);
                } else {
                    throw "ColorPicker does not use Kendo for input";
                }
            },

            get_input: function() { return $(this._input); },
            get_element: function() { return this.get_input(); },
            get_container: function() { return $(this._container); }
        };
        function init() {
            var containers = $('[data-tpc-role="color-picker-container"]');

            if (!containers || containers.length < 1)
                return;

            for (var i = 0; i < containers.length; i++) {
                var container = $(containers[i]);
                var input = container.find('[data-tpc-role="color-picker-input"]');
                var id = container.attr("data-tpc-id");

                tpc.$create(id, container, new ColorPicker(container, input, id));
            }
        }
        init();
    });
}(jQuery));