﻿(function () {
    var simpleViewModule = angular.module('simpleViewModule', ['designer', 'tpcHelpers']);
    angular.module('designer').requires.push('expander', 'simpleViewModule');

    simpleViewModule.controller('SimpleCtrl', ['$scope', 'propertyService', function ($scope, propertyService) {
        $scope.feedback.showLoadingIndicator = true;

        $scope.defaultValue = null;
        $scope.initialized = false;
        $scope.designerState = {};

        $scope.supportedAttributes = ["String"];
        $scope.attributeOptions = [];
        
        $scope.selectedAttributeChanged = function(newAttr) {
            var attr = getAttributeData(newAttr);
            var attributeChanged = $scope.properties.Model.AttributeName.PropertyValue !== newAttr;
            if (attr) {
                $scope.properties.Model.AttributeType.PropertyValue = attr.AttributeType;
                if (attributeChanged || $scope.properties.Model.MetaField.Title.PropertyValue === $scope.properties.Model.DefaultTitle.PropertyValue) {
                    $scope.properties.Model.MetaField.Title.PropertyValue = attr.OptionText;
                }
                $scope.properties.Model.ReadOnly.PropertyValue = (attr.ReadOnly || false) ? "True" : "False";
                if (attr.ReadOnly) {
                    $scope.designerState.readOnlyDisabled = true;
                } else {
                    $scope.designerState.readOnlyDisabled = false;
                }
            }
        };
               
        var getAttributeData = function(name) {
            var attr = $($scope.attributeOptions).filter(function (a, b) { return b.OptionValue === name; });
            if (attr && attr.length) {
                attr = $(attr).eq(0)[0];
                return attr;
            }
            return null;
        }

        var getAttributeOptions = function() {
            var widgetContext = sitefinity.pageEditor.widgetContext;
            var queryAttributeOptions = function(entityLogicalName) {
                var attrUrl = "/PortalConnectorMvc/Services/Designer/AttributeService/GetAttributes/" + entityLogicalName + "?attributeTypes=" +
                    $scope.supportedAttributes.join(';');
                var options = [];
                $.ajax(attrUrl, {
                    async: false,
                    dataType: "json",
                    headers: sitefinity.pageEditor.widgetContext.PropertyValueCulture ? { "Accept-Language": sitefinity.pageEditor.widgetContext.PropertyValueCulture } : {},
                }).done(function(data) {
                    options = data;
                }).fail(function(err) {
                    console.log(err);
                });
                return options;
            }
            if (widgetContext) {
                var fcUrl = "/PortalConnectorMvc/Services/Designer/RelationshipService/GetParentFormConfig/" + widgetContext.Id;
                $.ajax(fcUrl, {
                    async: false,
                    dataType: "json",
                    context: $scope,
                    headers: sitefinity.pageEditor.widgetContext.PropertyValueCulture ? { "Accept-Language": sitefinity.pageEditor.widgetContext.PropertyValueCulture } : {},
                }).done(function(data) {
                    if (data && data.LogicalName) {
                        this.attributeOptions = queryAttributeOptions(data.LogicalName);
                    }
                }).fail(function(err) {
                    console.log(err);
                });
            };
        };

        var onPostPropertiesInitialized = function () {
            $scope.$watch('designerState', function (newData, oldData) {
                if (newData !== oldData) {
                    $scope.designerState = newData;
                    
                    if ($scope.designerState.selectedAttribute) {
                        $scope.properties.Model.AttributeName.PropertyValue = $scope.designerState.selectedAttribute;
                        if ($("#developerName").length > 0) {
                            $scope.properties.Model.FieldName.PropertyValue = $scope.properties.Model.AttributeName.PropertyValue;
                        }
                    }
                }
            }, true);
            $scope.initialized = true;
        };

        var onGetPropertiesSuccess = function (data) {
            if (data) {
                $scope.properties = propertyService.toHierarchyArray(data.Items);

                getAttributeOptions();

                if ($scope.properties.Model.ValidatorDefinition.MaxLength.PropertyValue === '0') {
                    $scope.properties.Model.ValidatorDefinition.MaxLength.PropertyValue = '';
                }

                $scope.designerState.selectedAttribute = $scope.properties.Model.AttributeName.PropertyValue;

                var selectedAttrData = getAttributeData($scope.designerState.selectedAttribute);
                if (selectedAttrData) $scope.designerState.readOnlyDisabled = selectedAttrData.ReadOnly;
                
                $scope.properties.Model.MetaField.DefaultValue.PropertyValue = $scope.defaultValue;
            }
        };

        propertyService.get()
            .then(onGetPropertiesSuccess)
            .catch(function (data) {
                $scope.feedback.showError = true;
                if (data) {
                    $scope.feedback.errorMessage = data.Detail;
                }
            })
            .then(function () {
                $scope.feedback.savingHandlers.push(function () {
                    if (!$scope.designerState.selectedAttribute) {
                        throw "Please select an attribute before saving.";
                    }
                });
            })
            .then(onPostPropertiesInitialized)
            .finally(function () {
                $scope.feedback.showLoadingIndicator = false;
            });
    }]);
    
})();